@@ -1,13 +1,10 @@
 # Authors: Stephane Gaiffas <stephane.gaiffas@gmail.com>
 # License: BSD 3 clause
-
 from math import exp
-
 import numpy as np
 from numba import njit
-from numba.types import float32, boolean, uint32, void
 from numpy.random import uniform
-
+from .types import float32, boolean, uint32, void
 from .node_methods import (
     node_get_child,
     node_compute_split_time,
@@ -19,7 +16,7 @@ from .node_methods import (
     node_update_weight_tree,
 )
 from .tree import TreeClassifier
-from .utils import sample_discrete
+from .utils import sample_discrete, get_type
 
 
 # TODO: an overall task is to minimize the O(#n_features) complexity: pass few
@@ -27,7 +24,7 @@ from .utils import sample_discrete
 # TODO: write all the docstrings
 
 
-@njit(uint32(TreeClassifier.class_type.instance_type, uint32))
+@njit(uint32(get_type(TreeClassifier), uint32))
 def tree_go_downwards(tree, idx_sample):
     # We update the nodes along the path which leads to the leaf containing
     # x_t. For each node on the path, we consider the possibility of
@@ -35,6 +32,7 @@ def tree_go_downwards(tree, idx_sample):
     # Index of the root is 0
     idx_current_node = 0
     x_t = tree.samples.features[idx_sample]
+    nodes = tree.nodes
 
     if tree.iteration == 0:
         # If it's the first iteration, we just put x_t in the range of root
@@ -45,6 +43,7 @@ def tree_go_downwards(tree, idx_sample):
             # If it's not the first iteration (otherwise the current node
             # is root with no range), we consider the possibility of a split
             split_time = node_compute_split_time(tree, idx_current_node, idx_sample)
+
             if split_time > 0:
                 # We split the current node: because the current node is a
                 # leaf, or because we add a new node along the path
@@ -75,9 +74,9 @@ def tree_go_downwards(tree, idx_sample):
                 # Update the current node
                 node_update_downwards(tree, idx_current_node, idx_sample, True)
 
-                left = tree.nodes.left[idx_current_node]
-                right = tree.nodes.right[idx_current_node]
-                depth = tree.nodes.depth[idx_current_node]
+                left = nodes.left[idx_current_node]
+                right = nodes.right[idx_current_node]
+                depth = nodes.depth[idx_current_node]
 
                 # Now, get the next node
                 if is_right_extension:
@@ -97,14 +96,14 @@ def tree_go_downwards(tree, idx_sample):
                 # There is no split, so we just update the node and go to
                 # the next one
                 node_update_downwards(tree, idx_current_node, idx_sample, True)
-                is_leaf = tree.nodes.is_leaf[idx_current_node]
+                is_leaf = nodes.is_leaf[idx_current_node]
                 if is_leaf:
                     return idx_current_node
                 else:
                     idx_current_node = node_get_child(tree, idx_current_node, x_t)
 
 
-@njit(void(TreeClassifier.class_type.instance_type, uint32))
+@njit(void(get_type(TreeClassifier), uint32))
 def tree_go_upwards(tree, leaf):
     idx_current_node = leaf
     if tree.iteration >= 1:
@@ -118,7 +117,7 @@ def tree_go_upwards(tree, leaf):
             idx_current_node = tree.nodes.parent[idx_current_node]
 
 
-@njit(void(TreeClassifier.class_type.instance_type, uint32))
+@njit(void(get_type(TreeClassifier), uint32))
 def tree_partial_fit(tree, idx_sample):
     leaf = tree_go_downwards(tree, idx_sample)
     if tree.use_aggregation:
@@ -126,40 +125,41 @@ def tree_partial_fit(tree, idx_sample):
     tree.iteration += 1
 
 
-@njit(uint32(TreeClassifier.class_type.instance_type, float32[::1]))
+@njit(uint32(get_type(TreeClassifier), float32[::1]))
 def tree_get_leaf(tree, x_t):
     # Find the index of the leaf that contains the sample. Start at the root.
     # Index of the root is 0
     node = 0
     is_leaf = False
+    nodes = tree.nodes
     while not is_leaf:
-        is_leaf = tree.nodes.is_leaf[node]
+        is_leaf = nodes.is_leaf[node]
         if not is_leaf:
-            feature = tree.nodes.feature[node]
-            threshold = tree.nodes.threshold[node]
+            feature = nodes.feature[node]
+            threshold = nodes.threshold[node]
             if x_t[feature] <= threshold:
-                node = tree.nodes.left[node]
+                node = nodes.left[node]
             else:
-                node = tree.nodes.right[node]
+                node = nodes.right[node]
     return node
 
 
-@njit(
-    void(TreeClassifier.class_type.instance_type, float32[::1], float32[::1], boolean)
-)
+@njit(void(get_type(TreeClassifier), float32[::1], float32[::1], boolean))
 def tree_predict(tree, x_t, scores, use_aggregation):
+    nodes = tree.nodes
     leaf = tree_get_leaf(tree, x_t)
     if not use_aggregation:
         node_predict(tree, leaf, scores)
         return
     current = leaf
+    # Allocate once and for all
     pred_new = np.empty(tree.n_classes, float32)
     while True:
-        if tree.nodes.is_leaf[current]:
+        if nodes.is_leaf[current]:
             node_predict(tree, current, scores)
         else:
-            weight = tree.nodes.weight[current]
-            log_weight_tree = tree.nodes.log_weight_tree[current]
+            weight = nodes.weight[current]
+            log_weight_tree = nodes.log_weight_tree[current]
             w = exp(weight - log_weight_tree)
             # Get the predictions of the current node
             node_predict(tree, current, pred_new)
@@ -169,4 +169,4 @@ def tree_predict(tree, x_t, scores, use_aggregation):
         if current == 0:
             break
         # And now we go up
-        current = tree.nodes.parent[current]
+        current = nodes.parent[current]
